@@ -608,7 +608,7 @@ nautilus_application_startup (NautilusApplication *application,
 				/* FIXME bugzilla.gnome.org 42538: When can this happen? */
 				message = _("Nautilus can't be used now, due to an unexpected error.");
 				detailed_message = _("Nautilus can't be used now, due to an unexpected error "
-						     "from Bonobo when attempting to locate the factory."
+						     "from Bonobo when attempting to locate the factory. "
 						     "Killing bonobo-activation-server and restarting Nautilus may help fix the problem.");
 			} else {
 				num_failures++;
@@ -1139,7 +1139,8 @@ nautilus_application_present_spatial_window_with_selection (NautilusApplication
 #endif
 
 			gtk_window_present (GTK_WINDOW (existing_window));
-			if (new_selection) {
+			if (new_selection &&
+			    existing_window->content_view != NULL) {
 				nautilus_view_set_selection (existing_window->content_view, new_selection);
 			}
 
@@ -1473,6 +1474,9 @@ save_session_to_file (void)
 		g_free (filename);
 		filename = NULL;
 	}
+	
+	xmlSaveClose(ctx);
+	close (fd);	
 
  out:
 	xmlFreeDoc (doc);
